/*
 * Copyright (c) 2006-2009 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
/**
 ** GPSGPSTaskCodels.c
 **
 ** Codels called by execution task GPSGPSTask
 **
 ** Author: F Camps
 ** Date: 07/08/2006
 **
 **/

#include <unistd.h>

#include <portLib.h>

#include "GPSConst.h"

#include "server/GPSHeader.h"
#include <libeuler/pomEuler.h>

#include <GPSNovatel/GPSNovatel.h>
#include <GPSNovatel/sampleAPI.h>

#include "h2mathLib.h"
#include "h2evnLib.h"
#include "h2timeLib.h"

static int connected;
static int serialFd;
static POSTER_ID headingPosterId;
static POSTER_ID attitudePosterId;
static POSTER_ID framePosterId;
static STATUS exportPomME(int *);

static FILE *GPSLogFile;

/*------------------------------------------------------------------------
 * writeGPSCommand
 *
 * Description: write a "ascii" command to the GPS, no blocking send.
 *
 * Reports:      OK
 *              S_GPS_FAILED_WRITE
 */
STATUS
writeGPSCommand(int fd, char* command, int* report)
{
  int retWrite = putGPSCommand(fd, command);

  if(retWrite == 0 || retWrite == -1) {
    *report = S_GPS_FAILED_WRITE;
    fprintf(stderr, "[GPS-genom] write error to : %s\n",
            SDI_F->gpsInit.comNumber);
    return ERROR;
  }
  return OK;
}


STATUS
openGPSSerial(GPS_INIT *gpsInit, int* report)
{
  
  // if the serial is already initialized, close it
  if (serialFd)
    close(serialFd);
  
  /* Open the serial port COM1 */
  if(( serialFd = initGPS(gpsInit->comNumber, gpsInit->VMIN,  gpsInit->VTIME, gpsInit->comSpeedCOM1 )) < 0) {
    *report = S_GPS_CANNOT_CONNECT;
    return ERROR;
  }
  return OK;
}

STATUS
configureGPS(GPS_INIT *gpsInit, int* report)
{

  char speedCOM[30];
  sprintf(speedCOM, "COM COM2 %d\n",  gpsInit->comSpeedCOM2);
  if (writeGPSCommand(serialFd, speedCOM, report) == ERROR)
    return ERROR;

  // this config is for GPS in RTK mode with RTCM messages
  //  received on COM1 
  if (writeGPSCommand(serialFd, "INTERFACEMODE COM2 RTCMNOCR NONE OFF\n",
                      report) == ERROR)
    return ERROR;

  if (writeGPSCommand(serialFd, "UNLOGALL COM1\n",
                      report) == ERROR)
    return ERROR;


  /* ask UTM log in binary on COM1 */
  char logUTMCommand[50];
  sprintf(logUTMCommand, "LOG COM1 BESTUTMb ontime %f\n", GPS_UTM_PERIOD);
  if (writeGPSCommand(serialFd, logUTMCommand, report)
      == ERROR)
    return ERROR;
  
  sprintf(logUTMCommand, "LOG COM1 BESTVELb ontime %f\n", GPS_VEL_PERIOD);
  if (writeGPSCommand(serialFd, logUTMCommand, report)
      == ERROR)
    return ERROR;

  if (gpsInit->heading) {
    snprintf(logUTMCommand, sizeof(logUTMCommand), "LOG COM1 HEADINGb onnew\n");
    if (writeGPSCommand(serialFd, logUTMCommand, report)
	== ERROR)
      return ERROR;
  }
  return OK;
}

/*------------------------------------------------------------------------
 *
 * startGPSTask  --  Initialization codel (fIDS, ...)
 *
 * Description:
 *
 * Returns:    OK or ERROR
 */
STATUS
startGPSTask(int *report)
{
#ifdef DEBUG
  printf("************ startGPSTask 1\n");
#endif

  return OK;
}

STATUS
endGPSTask(void)
{
  int report;

  if (!connected)
    return OK;

  /* remove all current logs*/
  if (writeGPSCommand(serialFd, "UNLOGALL COM1\n", &report) == ERROR)
    fprintf(stderr, "[GPS-genom] WARNING : cannot unlog all logs ! \n");

  /* end the log function in the GPSNovatel lib*/
  endGPS(SDI_F->log.logToFile);

  return OK;
}

/* GPSConnectExec  - Connect to the GPS 
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
GPSConnectExec(GPS_INIT *gpsInit, int *report)
{

  if (openGPSSerial(gpsInit,report)!=OK) {
    connected = GPS_FALSE;
    close(serialFd);
    fprintf(stderr, "[GPS-genom] ERROR : cannot open  GPS ! \n");    
    return FAIL;
  }
  
  if (configureGPS(gpsInit,report)!=OK) {
    connected = GPS_FALSE;
    close(serialFd);
    fprintf(stderr, "[GPS-genom] ERROR : cannot configure GPS ! \n");    
    return FAIL;
  }
  
  connected = GPS_TRUE;
  return ETHER;
}

/* GPSDisconnectExec  -  Disconnect from the GPS
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
GPSDisconnectExec(int *report)
{
  // if the serialFd is not initialized
  if (!serialFd || !connected)
    return ETHER;
  //
  if (writeGPSCommand(serialFd, "UNLOGALL COM1\n",
                      report) == ERROR)
    return FAIL;
  
  connected = GPS_FALSE;
  return END;
}

/* GPSReconnectExec  - Reconnect to the GPS
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
GPSReconnectExec(int *report)
{
  // if the serialFd is initialized and connected=GPS_TRUE
  if (serialFd && connected)
    return ETHER;

  //
  if (writeGPSCommand(serialFd, "UNLOGALL COM1\n",
                      report) == ERROR)
    return FAIL;
  
  connected = GPS_FALSE;
  return END;
}

/* GPS_Data_Read  -  codel EXEC of GSP_DATA
   Returns:  EXEC END ETHER FAIL ZOMBIE */
STATUS
GPSTaskPerm(int *report)
{
  if (!connected)
    return OK;

  // tell if the utm pos is updated, used to decide on exporting MEPos
  int utmUpdated=FALSE;
  // tell if the pos pos is updated, used to decide on logging in file
  int needToLog=FALSE;
  int ret=0, nbTrame=0, ind=0;

  /* get buffered data from GPS */
  if((ret = getGPSData(serialFd, &nbTrame)) < 0) {
    /* pb of read */
    *report =  S_GPS_FAILED_READ;
    return OK;
  }

  /*for debug purpose */
  SDI_F->debug.nbTrame=nbTrame;

  if(nbTrame > 0) {
    /*
     * on parcourt tous les messages recus du GPS
     * et on traite seulement
     * ceux qui nous interessent
     */
    for(ind=0; ind<nbTrame; ind++) {
      switch (USR_TRAME[ind].BINARY_HEADER.number) {
      case  BESTPOSb:
	/* recuperation des informations NMEA avec format WGS84 */
        SDI_F->date_POS = USR_TRAME[ind].timev.tv_sec + 
                          USR_TRAME[ind].timev.tv_usec*1e-6;
        SDI_F->POS =
          USR_TRAME[ind].TRAME_TYPE.strOEM4BESTPOSb_LOG;
        needToLog=TRUE;
	break;
      case BESTUTMb:
	/* recuperation des informations UTM */
        // sometimes the gps cannot get a position and returns a message
        // with zeros almost everywhere, so just ignore them
        if (USR_TRAME[ind].TRAME_TYPE.strOEM4BESTUTMb_LOG.posType != 0 && 
            USR_TRAME[ind].TRAME_TYPE.strOEM4BESTUTMb_LOG.longitudZoneNumber != 0 &&
            USR_TRAME[ind].TRAME_TYPE.strOEM4BESTUTMb_LOG.solStatus == 0)
        {
          SDI_F->date_UTM = USR_TRAME[ind].timev.tv_sec + 
                            USR_TRAME[ind].timev.tv_usec*1e-6;
          SDI_F->UTM =
            USR_TRAME[ind].TRAME_TYPE.strOEM4BESTUTMb_LOG;
        
          utmUpdated=TRUE;
        }
	break;
      case BESTVELb:
        /* recuperation des informations de velocite */
        SDI_F->date_vel = USR_TRAME[ind].timev.tv_sec + 
                          USR_TRAME[ind].timev.tv_usec*1e-6;
        SDI_F->vel =
          USR_TRAME[ind].TRAME_TYPE.strOEM4BESTVELb_LOG;
	break;
      case HEADINGb:
	/* Heading from an OEM6 capable device */
	SDI_F->date_heading = USR_TRAME[ind].timev.tv_sec +
	  USR_TRAME[ind].timev.tv_usec*1e-6;
	SDI_F->heading = USR_TRAME[ind].TRAME_TYPE.strOEM6HEADINGb_LOG;
	break;
      } /* switch */
    } /* for */

    // LOG
    if (SDI_F->log.logToFile && needToLog)
      logData();
  }

  // EXPORT TO POM
  STATUS exportStatus=OK;
  /* if the received information is UTM, export to pom */
  if( utmUpdated ) {
    if ((exportStatus=exportPomME(report))!=OK)
      return exportStatus;
  }

  return OK;
}

/**
 * Reads the current UTM position in SDI_F->UTM and updates the MEPos poster
 * accordingly.
 */
STATUS
exportPomME(int *report)
{
  POM_EULER_V current;
  memset(&current, 0, sizeof(POM_EULER_V));

  // fullfill current POM_EULER_V structure with the GPS one
  // must fullfill all infos (x,y,z,y,p,r) for the transformation from
  // gps_to_origin to main_to_origin
  current.euler.x = SDI_F->UTM.easting  - SDI_F->posRef.x;
  current.euler.y = SDI_F->UTM.northing - SDI_F->posRef.y;
  current.euler.z = SDI_F->UTM.height   - SDI_F->posRef.z;
  // variance on x
  current.var.vxx = SDI_F->UTM.eastingDev * SDI_F->UTM.eastingDev;
  // variance on y
  current.var.vyy = SDI_F->UTM.northingDev * SDI_F->UTM.northingDev;

  current.euler.yaw   = SDI_F->pos.main.euler.yaw;
  current.euler.pitch = SDI_F->pos.main.euler.pitch;
  current.euler.roll  = SDI_F->pos.main.euler.roll;

  /* Set the confidence according to the GPS position type
   * Rflex exports 0.2, and we saw that it derives not that much
   * posType =
   *    50 : NARROW_INT
   *    49 : WIDE_INT
   *    48 : L1_INT
   *    34 : NARROW_FLOAT
   * See Novatel user manual Vol 2 p.162-163 for the values
   * of posType
   *//*
  if (SDI_F->UTM.posType < 34)
    SDI_F->pos.confidence = 0.0;
  else if (SDI_F->UTM.posType < 48)
    SDI_F->pos.confidence = 0.1;
  else if (SDI_F->UTM.posType == 48)
    SDI_F->pos.confidence = 0.3;
  else if (SDI_F->UTM.posType == 49)
    SDI_F->pos.confidence = 0.5;
  else
    SDI_F->pos.confidence = 1.0;
*/
/*
FIXME: hack
pom doesn't fuse correctly when confidence different from 0
And posType is not a good indicator of precision, narrow_int can go up to more 
than one meter, and wide_int can go down to less than 5 cm
*/
const double thres = 0.15;
if (SDI_F->UTM.eastingDev <= thres && SDI_F->UTM.northingDev <= thres && SDI_F->UTM.heightDev <= 0.15)
	SDI_F->pos.confidence = 1.0; else SDI_F->pos.confidence = 0.0;

  // fullfill the SDI variable posQuality
  (SDI_F->posQuality).northing_stddev = SDI_F->UTM.northingDev;
  (SDI_F->posQuality).easting_stddev = SDI_F->UTM.eastingDev;
  (SDI_F->posQuality).pom_confidence = SDI_F->pos.confidence;

  /**
   * WARN: do that BEFORE setting the attitude, since the attitude
   * is already in the main frame
   */
  if (framePosterId) {
    // read the pom frame position poster
    POM_SENSOR_POS framePos;
    if (GPSPOM_SENSOR_POSPosterRead(framePosterId, &framePos) == ERROR) {
      printf("[GPS-genom] ERROR : cannot read pom tagging poster %s \n",SDI_F->pomTagging.pomPoster.str);
      *report = S_GPS_CANNOT_READ_POM;
      framePosterId = NULL;
      return ERROR;
    }

    /* get the time tags */
#ifdef DEBUG
    printf("POM tagging date : %d \n",framePos.date);
#endif
    /* for debugging purpose, copy this frame to SDI  */
    SDI_F->debug.pomTaggingFrame=framePos;
    // tag the current position with timestamp from pom frame position
    SDI_F->pos.date1 = framePos.date;
    /**
     * compute the MEposter main frame from current with transformation info from pom
     * Compose framePos.sensorToMain with current, store the result in SDI_F->pos.main
     */

#ifdef DEBUG
    printf("[GPS-genom] pomGPSFramePos sensorToMain : %f %f %f \n",framePos.sensorToMain.euler.x,framePos.sensorToMain.euler.y,framePos.sensorToMain.euler.z);
#endif

    pomEulerVCompIn(&SDI_F->pos.main, pomEulerVInvert(&framePos.sensorToMain), &current);

#ifdef DEBUG
    printf("[GPS-genom] pos exported to pom in MEPos : %f %f %f \n",SDI_F->pos.main.euler.x,SDI_F->pos.main.euler.y,SDI_F->pos.main.euler.z);
#endif

  } else {
    SDI_F->pos.main = current;
  }

  /**
   * try to read attitude poster for additionnal infos (pitch/roll/yaw)
   * set by the request SetAttitudePoster
   */
  if (attitudePosterId) {
    POM_SENSOR_POS attitude;
    if (GPSPOM_SENSOR_POSPosterRead(attitudePosterId, &attitude)
        == ERROR) {
      printf("[GPS-genom] Error : cannot read attitude poster\n");
      *report = S_GPS_CANNOT_READ_ATTITUDE;
      return OK;
    }

    SDI_F->pos.main.euler.yaw   = attitude.sensorToMain.euler.yaw;
    SDI_F->pos.main.euler.pitch = attitude.sensorToMain.euler.pitch;
    SDI_F->pos.main.euler.roll  = attitude.sensorToMain.euler.roll;
    SDI_F->pos.main.var.voo     = attitude.sensorToMain.var.voo;
    SDI_F->pos.main.var.vop     = attitude.sensorToMain.var.vop;
    SDI_F->pos.main.var.vpp     = attitude.sensorToMain.var.vpp;
    SDI_F->pos.main.var.vpr     = attitude.sensorToMain.var.vpr;
    SDI_F->pos.main.var.vrr     = attitude.sensorToMain.var.vrr;
  }

  // try to get additionnal yaw info in heading poster
  if (headingPosterId) {
    POM_ME_POS heading;
    if (GPSPOM_ME_POSPosterRead(headingPosterId, &heading)
        == ERROR) {
      printf("[GPS-genom] Error : cannot read heading poster poster\n");
      *report = S_GPS_CANNOT_READ_HEADING;
      return OK;
    }
    SDI_F->pos.main.euler.yaw   = heading.main.euler.yaw;
    SDI_F->pos.main.var.voo     = heading.main.var.voo;
  }

#ifdef DEBUG
  /* for debugging purpose */
  H2TIME date;
  posterIoctl(GPS_MEPOS_POSTER_ID, FIO_GETDATE, (char *)&date);
  SDI_F->debug.dateWriteMEPoster=date.ntick;
#endif

  /**
   * for debug purpose : just tell GPS to have bad confidence and bahavior : to see the behavior of pom
   */
  if (SDI_F->debug.stopExportingToPom==TRUE) {
/*     double rd = ((double)rand())/RAND_MAX;
 *     if (rd < 0.2) {
 */
      // set the confidence of GPS ME to 0.0
      SDI_F->pos.confidence=0.0;
      // set x,y with error
      //
      SDI_F->pos.main.euler.x=SDI_F->pos.main.euler.x+1;
      SDI_F->pos.main.euler.y=SDI_F->pos.main.euler.y+1;
      SDI_F->pos.main.euler.z=SDI_F->pos.main.euler.z+1;
/*     }
 */
  }

  /* write the MEPos poster from SDI POM_SENSOR_POS previously computed */
  if (GPSMEPosPosterWrite (GPS_MEPOS_POSTER_ID,
			   (GPS_MEPOS_POSTER_STR *)&(SDI_F->pos)) != OK) {
    printf("[GPS-genom]  Could not update MEPos poster\n");
  }

  // Log
  if (GPSLogFile) {
  /* get date */
  H2TIMESPEC tv;
  h2GetTimeSpec(&tv);

  /* save date */
  if (!fprintf(GPSLogFile, "%lf ",
               tv.tv_sec + tv.tv_nsec*1.0E-9)) {
    GPSLogFile = 0;
    *report = S_GPS_CANNOT_WRITE_FILE;
    return ETHER;
  }

  /* save UTM pos */
  int centimetric=0;
  if (SDI_F->UTM.posType>=50)
    centimetric=1;
   
/*  fprintf (GPSLogFile, "%f %f %f %f %f %f %d\n",
           SDI_F->pos.main.euler.x, SDI_F->pos.main.euler.y, SDI_F->pos.main.euler.z, SDI_F->UTM.northingDev, SDI_F->UTM.eastingDev, SDI_F->UTM.heightDev,centimetric);
*/
  fprintf (GPSLogFile, "%f %f %f %f %f %f %d %d %d %d %d %f %f %f %f %d %d %d %d %d %f %f %d %d %d %d %d %d %d %d %d %d %d %f %f %f %f %f\n",
					 SDI_F->pos.main.euler.x, SDI_F->pos.main.euler.y, SDI_F->pos.main.euler.z,
					 SDI_F->UTM.northingDev, SDI_F->UTM.eastingDev, SDI_F->UTM.heightDev,
					 centimetric, SDI_F->UTM.solStatus, SDI_F->UTM.posType, SDI_F->UTM.longitudZoneNumber, SDI_F->UTM.latitudinalZoneLetter,
					 SDI_F->UTM.northing, SDI_F->UTM.easting, SDI_F->UTM.height, SDI_F->UTM.undulation, SDI_F->UTM.datumID,
					 SDI_F->UTM.baseStationID[0], SDI_F->UTM.baseStationID[1], SDI_F->UTM.baseStationID[2], SDI_F->UTM.baseStationID[3],
					 SDI_F->UTM.differentialAge, SDI_F->UTM.solutionAge, SDI_F->UTM.nbSatTracked, SDI_F->UTM.nbSatTracked,
					 SDI_F->UTM.nbGPSL1Used, SDI_F->UTM.nbGPSL1, SDI_F->UTM.nbGPSL2,
					 SDI_F->UTM.reserved[0], SDI_F->UTM.reserved[1], SDI_F->UTM.reserved[2], SDI_F->UTM.reserved[3],
					 SDI_F->vel.solStatus, SDI_F->vel.velocityType, SDI_F->vel.latency, SDI_F->vel.age,
					 SDI_F->vel.horizontalSpeed, SDI_F->vel.actualDirection, SDI_F->vel.verticalSpeed
					 );
  }

  return OK;
}


/*------------------------------------------------------------------------
 * SendCommand
 *
 * Description:
 *
 * Reports:      OK
 *              S_GPS_ORDER_NOT_ALLOWED
 */

/* SendCommandFunc  -  codel EXEC of SendCommand
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
GPSSendCommandExec(GEN_STRING128* order, int *report)
{
  char command[1024];
  sprintf(command,"%s\n",order->name);
  if(strlen(order->name) > 0) {
    if (writeGPSCommand(serialFd, command, report) == ERROR)
      return ETHER;
  }
  return ETHER;
}

/*------------------------------------------------------------------------
 * SetPom
 *
 * Description:
 *
 * Reports:      OK
 */

/* SetPomExec  -  codel EXEC of SetPom
 * Set the pom time tagging, find the pom poster for getting tagging
 Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SetPomExec(GPS_POM_TAGGING *pomTagging, int *report)
{
  if (pomTagging->flag) {
    if (GPSPOM_SENSOR_POSPosterFind(pomTagging->pomPoster.str, &framePosterId)
        == ERROR) {
#ifdef DEBUG
      printf("[GPS-genom] Cannot read pom tagging poster %s \n",pomTagging->pomPoster.str);
#endif
      *report = S_GPS_CANNOT_READ_POM;
      return ETHER;
    }
  } else
    framePosterId = NULL;

  return ETHER;
}

/*------------------------------------------------------------------------
 * SetHeadingPoster
 *
 * Description:
 *
 * Reports:      OK
 *              S_GPS_NOT_FOUND
 */

/* SetHeadingPosterExec  -  codel EXEC of SetHeadingPoster
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SetHeadingPosterExec(GPS_STRING *headingPosterName, int *report)
{
  if (! *headingPosterName->str)
    headingPosterId = NULL;
  else if (GPSPOM_ME_POSPosterFind(headingPosterName->str,
                                   &headingPosterId) == ERROR)
    *report = S_GPS_NOT_FOUND;
  return ETHER;
}

/*------------------------------------------------------------------------
 * SetAttitudePoster
 *
 * Description:
 *
 * Reports:      OK
 *              S_GPS_NOT_FOUND
 */

/* SetAttitudePosterExec  -  codel EXEC of SetAttitudePoster
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SetAttitudePosterExec(GPS_STRING *attitudePosterName, int *report)
{
  if (! *attitudePosterName->str)
    attitudePosterId = NULL;
  else if (GPSPOM_SENSOR_POSPosterFind(attitudePosterName->str,
                                       &attitudePosterId) == ERROR)
    *report = S_GPS_NOT_FOUND;
  return ETHER;
}

/*------------------------------------------------------------------------
 * Log
 *
 * Description:
 *
 * Reports:      OK
 *              S_GPS_CANNOT_OPEN_FILE
 *              S_GPS_CANNOT_WRITE_FILE
 */



/* GPSLogStart  -  codel START of Log
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
GPSLogStart(GEN_STRING128 *fileName, int *report)
{

  if (!(GPSLogFile = fopen(fileName->name, "a"))) {
    printf ("[GPS-genom] cannot open gps log file %s\n", fileName->name);
    *report = S_GPS_CANNOT_OPEN_FILE;
    return ETHER;
  }

  printf ("[GPS-genom] log file %s found\n", fileName->name);
//  fprintf (GPSLogFile, "[GPS-genom] # Time (sec) utm_x utm_y utm_z x_sd y_sd z_sd mode\n");

  fprintf (GPSLogFile, "time(s) robot_utm_x robot_utm_y robot_utm_z x_sd y_sd z_sd mode solStatus \
					 posType longitudZoneNumber latitudinalZoneLetter \
					 sensor_northing sensor_easting sensor_height undulation datumID \
					 baseStationID[0] baseStationID[1] baseStationID[2] baseStationID[3] \
					 differentialAge solutionAge nbSatTracked nbSatTracked nbGPSL1Used nbGPSL1 nbGPSL2 \
					 reserved[0] reserved[1] reserved[2] reserved[3] solStatus velocityType latency age horizontalSpeed actualDirection verticalSpeed\n");

	
  return EXEC;
}

/* GPSLogExec  -  codel EXEC of Log
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
GPSLogExec(GEN_STRING128 *fileName, int *report)
{
	
  return EXEC;
}

/* GPSLogInter  -  codel END of Log
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
GPSLogInter(GEN_STRING128 *fileName, int *report)
{
  fclose(GPSLogFile);
  GPSLogFile = NULL;

  return ETHER;
}
