/*
 * Copyright (c) 2003-2011 CNRS/LAAS
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <unistd.h>

#include "serial.h"

#ifdef __sun
static void cfmakeraw(struct termios *);
static int cfsetspeed(struct termios *, speed_t);
#endif

int 
openSerial(const char *device)
{
	int fd, flags;
	struct termios t;

	fd = open(device, O_RDWR|O_NONBLOCK);
	if (fd < 0) {
		fprintf(stderr, "%s:%d: %s: %s\n", __FILE__, __LINE__,
		    device, strerror(errno));
		return -1;
	}
	if (tcgetattr(fd, &t) < 0) {
		fprintf(stderr, "%s:%d: tcgetattr: %s\n", __FILE__, __LINE__,
		    strerror(errno));
		return -1;
	}
	
	cfmakeraw(&t);
	t.c_cc[VMIN] = 10;
	t.c_cc[VTIME] = 10;
	t.c_cflag |= CRTSCTS;
	/* as configured on the receiver? */
	if (cfsetspeed(&t, B57600) < 0) {
		fprintf(stderr, "%s:%d: cfsetspeed: %s\n", __FILE__, __LINE__,
		    strerror(errno));
		return -1;
	}
	if (tcsetattr(fd, TCSAFLUSH, &t) < 0) {
		fprintf(stderr, "%s:%d: tcsetattr: %s\n", __FILE__, __LINE__,
		    strerror(errno));
		close (fd);
		return -1;
	}
	flags = fcntl(fd, F_GETFL, 0);
	if (flags < 0) {
		fprintf(stderr, "%s:%d: F_GETFL: %s\n", __FILE__, __LINE__,
		    strerror(errno));
		close (fd);
		return -1;
	}
	flags &= ~O_NONBLOCK;
	if (fcntl(fd, F_SETFL, flags) < 0) {
		fprintf(stderr, "%s:%d: F_SETFL: %s\n", __FILE__, __LINE__,
		    strerror(errno));
		close (fd);
		return -1;
	}
	
	return fd;
}

int
readSerial(int fd, char *buf, size_t len)
{
	int i = 0, n;
	char c;
	
	do {
		errno = 0;
		do {
			n = read(fd, &c, 1);
		} while (n < 0 && errno == EINTR && i < len);
		
		buf[i++] = c;
		if (c == '\n' || i == len) 
			break;
	} while (1);
	return i;
}

int 
writeSerial(int fd, char *buf, size_t len)
{
	/* for now */
	return write(fd, buf, len);
}

#ifdef __sun
/**
 ** These functions are missing in Solaris 
 **/
static void
cfmakeraw(struct termios *t)
{
        t->c_iflag &= ~(IMAXBEL|IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
        t->c_oflag &= ~OPOST;
        t->c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
        t->c_cflag &= ~(CSIZE|PARENB);
        t->c_cflag |= CS8;
        /* XXX set MIN/TIME */
}

static int
cfsetspeed(struct termios *t, speed_t speed)
{
	cfsetispeed(t, speed);
	cfsetospeed(t, speed);
        return (0);
}

#endif
